function rate=evol_trans_rate(id_trans,save_flag,flag_smooth,span)
% RATE=EVOL_TRANS_RATE(ID_TRANS,FLAG_SAV,FLAG_SMOOTH,SPAN)
%   Plot the evolution of the number of transitions with id number ID_TRANS per time unit.
%   If SAVE_FLAG=1, save figure.
%   If SAVE_FLAG=2, save data.
%   If SAVE_FLAG=3, save figure and data.
%   If FLAG_SMOOTH is set, the data are smoothed over time using SPAN value.

if ~exist('save_flag','var')
  save_flag=0;
end

if ~exist('flag_smooth','var')
  flag_smooth=0;
end


%% read header of TRANS.log
fprintf('reading TRANS.log\n');
fid=fopen('TRANS.log');
if (fid==-1)
  fprintf('file not found\n');
  return
end
textscan(fid,'# TRANSITIONS');
C=textscan(fid,'NB_TRANS_DB = %d');
nb_trans=C{1};
for i=1:nb_trans
  CTrans=textscan(fid,'TR%[^\n]s');
end

%% skip line of text
C=textscan(fid,'%[^\n]s');
%celldisp(C)

C=textscan(fid,'NB_TRANS_CEL = %d');
if size(C{1})>0
  nb_tcel=C{1};
else
  nb_tcel=0;
end

for i=1:nb_tcel
  CTransCel=textscan(fid,'TR1%[^\n]s');
end

%% skip line of text
C=textscan(fid,'%[^\n]s');
%celldisp(C)

C=textscan(fid,'NB_LINKS = %d');
if size(C{1})>0
  nb_link=C{1};
else
  nb_link=0;
end

if nb_link > 0
  for i=1:nb_link
    CLinks=textscan(fid,'LT%[^\n]s');
  end
  C=textscan(fid,'%[^\n]s');
  %celldisp(C)
end

%% read column headers
id=-1;
col_trans=-1;
nc=0;
%while (id~=id_trans) & (id~=nb_trans-1)
while (id~=nb_trans-1)
  C=textscan(fid,'TR( %d)');
  %celldisp(C)
  n=size(C{1},1);
  if n>0
    if (id < id_trans) & (id+n >= id_trans)
      col_trans=nc+(id_trans-id);
      fprintf('data for transition %d are located in column %d (+1)\n', id_trans, col_trans);
    end
    id=C{1}(n);
    nc=nc+n;
  end
  %C=textscan(fid,'%[BLK]s');
  %C=textscan(fid,'%[ ]BLK');
  C=textscan(fid,'B%[LK]');
  %keyboard
  %celldisp(C)
  if size(C{1},1)>0
    nc=nc+1;
  end
  %pause(1)
end

if nb_tcel>0
  id=-1;
  while (id~=nb_tcel-1)
    C=textscan(fid,'TR1( %d)');
    %celldisp(C);
    n=size(C{1},1);
    if n>0
      id=C{1}(n);
      nc=nc+n;
    end
  end
end

if nb_link>0
  id=-1;
  while (id~=nb_link-1)
    C=textscan(fid,'LT( %d)');
    %celldisp(C);
    n=size(C{1},1);
    if n>0
      id=C{1}(n);
      nc=nc+n;
    end
  end
end

fprintf('total number of columns: %d (+1)\n', nc);

%return

%% read trans data
C=textscan(fid,'%d','delimiter',':');
data0=cell2mat(C);
l=size(data0,1);
nb=l/(nc+1);
fprintf('number of records: %d\n', nb);
data=reshape(data0,nc+1,nb)';
transdata=data(:,col_trans+1);
fclose(fid);

%% read time data
fprintf('reading TIME.log\n');
fid=fopen('TIME.log');
if (fid==-1)
  fprintf('file not found\n');
  return
end
C=textscan(fid,'%[^\n]s');
%celldisp(C)
C=textscan(fid,'%f','delimiter',':dhms');
data0=cell2mat(C);
l=size(data0,1);
nb=nb+1;
c=l/nb;
data=reshape(data0,c,nb)';
time0=data(1,4);
timedata=data(2:nb,4);
dtimedata=data(2:nb,5);
fclose(fid);

%keyboard

avg=mean(transdata);
fprintf('average number of occurences per record: %f\n', avg);
dt=max(timedata)-time0;
if dt>0
  avg_norm=sum(transdata)/dt;
  fprintf('average number of occurences per time unit: %f\n', avg);
end
%keyboard

%% plot data
figure
hold on
lw=0.5;
if dt>0
  title(sprintf('Average nb of transitions (%d): %.2f t_0^{-1}', id_trans, avg_norm))
else
  title(sprintf('Number of transitions (%d)', id_trans))
end
rate=double(transdata)./dtimedata;
if ~flag_smooth
  plot(timedata,rate,'linewidth',lw)
else
  if ~exist('span','var')
    span=ceil(size(rate,1)/100);
  end
  fprintf('smooth rate on 2x%d records\n', span);
  rate_sm=smooth1(rate,span);
  plot(timedata,rate_sm,'linewidth',lw*2)
end
xlabel('Time (t_{0})')
ylabel('Overall rate (t_{0}^{-1})')
grid on

if save_flag
  str_sm='';
  if flag_smooth
    str_sm=sprintf('_sm%d',span);
  end
  if bitand(save_flag,1)
    figname=sprintf('EVOL_NB_TRANS_%d%s.png',id_trans,str_sm);
    fprintf('save figure into file: %s\n',figname);
    print(figname,'-dpng','-r250');
  end
  if bitand(save_flag,2)
    filename=sprintf('EVOL_NB_TRANS_%d%s.out',id_trans,str_sm);
    fprintf('save data into file: %s\n',filename);
    data=[timedata double(rate)];
    save(filename,'data','-ascii');
  end
end

return

